@@ -10,7 +10,7 @@
  *  -dvd_udf to find files
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: input_dvd.c,v 1.40 2001/04/08 16:57:47 sam Exp $
+ * $Id: input_dvd.c,v 1.47 2001/04/15 21:17:50 stef Exp $
  *
  * Author: Stphane Borel <stef@via.ecp.fr>
  *
@@ -35,9 +35,9 @@
 #include "defs.h"
 
 #ifdef HAVE_CSS
-#define MODULE_NAME dvd-css
+#define MODULE_NAME dvd
 #else /* HAVE_CSS */
-#define MODULE_NAME dvd-nocss
+#define MODULE_NAME dvdnocss
 #endif /* HAVE_CSS */
 #include "modules_inner.h"
 
@@ -350,29 +350,34 @@ static int DVDFindCell( thread_dvd_data_t * p_dvd )
 #define title \
         p_dvd->p_ifo->vts.title_unit.p_title[p_dvd->i_program_chain-1].title
 #define cell  p_dvd->p_ifo->vts.cell_inf
-
+		
     i_cell = p_dvd->i_cell;
     i_index = p_dvd->i_prg_cell;
 
+    if( i_cell >= cell.i_cell_nb )
+    {
+        return -1;
+    }
+
+
     while( ( ( title.p_cell_pos[i_index].i_vob_id !=
                    cell.p_cell_map[i_cell].i_vob_id ) ||
       ( title.p_cell_pos[i_index].i_cell_id !=
                    cell.p_cell_map[i_cell].i_cell_id ) ) &&
-           ( i_cell < cell.i_cell_nb ) )
+           ( i_cell < cell.i_cell_nb - 1 ) )
     {
         i_cell++;
     }
 
-    if( i_cell == cell.i_cell_nb )
-    {
-        intf_ErrMsg( "dvd error: can't find cell" );
-        return -1;
-    }
-    else
-    {
-        p_dvd->i_cell = i_cell;
-        return 0;
-    }
+intf_WarnMsg( 1, "FindCell: i_cell %d i_index %d found %d nb %d",
+                    p_dvd->i_cell,
+                    p_dvd->i_prg_cell,
+                    i_cell,
+                    cell.i_cell_nb );
+
+    p_dvd->i_cell = i_cell;
+    return 0;
+    
 #undef title
 #undef cell
 }
@@ -405,7 +410,15 @@ static int DVDFindSector( thread_dvd_data_t * p_dvd )
          p_dvd->p_ifo->vts.cell_inf.p_cell_map[p_dvd->i_cell].i_end_sector,
          title.p_cell_play[p_dvd->i_prg_cell].i_end_sector );
 
-//intf_WarnMsg( 3, "cell: %d sector1: 0x%x end1: 0x%x\nindex: %d sector2: 0x%x end2: 0x%x", p_dvd->i_cell, p_dvd->p_ifo->p_vts->c_adt.p_cell_inf[p_dvd->i_cell].i_ssector, p_dvd->p_ifo->p_vts->c_adt.p_cell_inf[p_dvd->i_cell].i_esector, p_dvd->i_prg_cell, p_pgc->p_cell_play_inf[p_dvd->i_prg_cell].i_entry_sector, p_pgc->p_cell_play_inf[p_dvd->i_prg_cell].i_lsector );
+    intf_WarnMsg( 1, "cell: %d sector1: 0x%x end1: 0x%x\n"
+                   "index: %d sector2: 0x%x end2: 0x%x", 
+        p_dvd->i_cell,
+        p_dvd->p_ifo->vts.cell_inf.p_cell_map[p_dvd->i_cell].i_start_sector,
+        p_dvd->p_ifo->vts.cell_inf.p_cell_map[p_dvd->i_cell].i_end_sector,
+        p_dvd->i_prg_cell,
+        title.p_cell_play[p_dvd->i_prg_cell].i_start_sector,
+        title.p_cell_play[p_dvd->i_prg_cell].i_end_sector );
+
 #undef title
 
     return 0;
@@ -413,7 +426,6 @@ static int DVDFindSector( thread_dvd_data_t * p_dvd )
 
 /*****************************************************************************
  * DVDChapterSelect: find the cell corresponding to requested chapter
- * When called to find chapter 1, also sets title size and end.
  *****************************************************************************/
 static int DVDChapterSelect( thread_dvd_data_t * p_dvd, int i_chapter )
 {
@@ -426,11 +438,15 @@ static int DVDChapterSelect( thread_dvd_data_t * p_dvd, int i_chapter )
     p_dvd->i_sector = 0;
 
     /* Search for cell_index in cell adress_table and initialize start sector */
-    DVDFindSector( p_dvd );
+    if( DVDFindSector( p_dvd ) < 0 )
+    {
+        intf_ErrMsg( "dvd error: can't select chapter" );
+        return -1;
+    }
 
     /* start is : beginning of vts vobs + offset to vob x */
     p_dvd->i_start = p_dvd->i_title_start +
-                     DVD_LB_SIZE * (off_t)( p_dvd->i_sector );
+	             DVD_LB_SIZE * (off_t)( p_dvd->i_sector );
 
     /* Position the fd pointer on the right address */
     p_dvd->i_start = lseek( p_dvd->i_fd, p_dvd->i_start, SEEK_SET );
@@ -460,6 +476,7 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
     u8                   i_mpeg;
     u8                   i_sub_pic;
     u8                   i;
+    int                  j;
     boolean_t            b_last;
 
     p_dvd = (thread_dvd_data_t*)p_input->p_plugin_data;
@@ -476,13 +493,19 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
         p_input->stream.p_selected_area =
                     p_input->stream.pp_areas[p_area->i_id];
 
-        /* title number: it is not vts nb! */
+        /* title number: it is not vts nb!,
+         * it is what appears in the interface list */
         p_dvd->i_title = p_area->i_id;
         p_dvd->p_ifo->i_title = p_dvd->i_title;
 
         /* ifo vts */
-        IfoTitleSet( p_dvd->p_ifo );
-        intf_WarnMsg( 2, "ifo info: vts initialized" );
+        if( IfoTitleSet( p_dvd->p_ifo ) < 0 )
+        {
+            intf_ErrMsg( "dvd error: fatal error in vts ifo" );
+            free( p_dvd );
+            p_input->b_error = 1;
+            return -1;
+        }
 
 #define vmg p_dvd->p_ifo->vmg
 #define vts p_dvd->p_ifo->vts
@@ -492,16 +515,36 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
         p_dvd->i_program_chain =
           vts.title_inf.p_title_start[p_dvd->i_vts_title-1].i_program_chain_num;
 
+/*        intf_WarnMsg( 1, "dvd: title %d vts_title %d pgc %d",
+                        p_dvd->i_title,
+                        p_dvd->i_vts_title,
+                        p_dvd->i_program_chain );
+*/
         /* css title key for current vts */
         if( p_dvd->b_encrypted )
         {
+            /* this one is vts number */
             p_dvd->p_css->i_title =
                     vmg.title_inf.p_attr[p_dvd->i_title-1].i_title_set_num;
             p_dvd->p_css->i_title_pos =
                     vts.i_pos +
                     vts.manager_inf.i_title_vob_start_sector * DVD_LB_SIZE;
-            CSSGetKey( p_dvd->p_css );
-            intf_WarnMsg( 2, "css info: vts key initialized" );
+
+            j = CSSGetKey( p_input->i_handle, p_dvd->p_css );
+            if( j < 0 )
+            {
+                intf_ErrMsg( "dvd error: fatal error in vts css key" );
+                free( p_dvd );
+                p_input->b_error = 1;
+                return -1;
+            }
+            else if( j > 0 )
+            {
+                intf_ErrMsg( "dvd error: css decryption unavailable" );
+                free( p_dvd );
+                p_input->b_error = 1;
+                return -1;
+            }
         }
     
         /*
@@ -516,19 +559,32 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
         p_dvd->i_cell = 0;
         p_dvd->i_prg_cell = -1 +
             vts.title_unit.p_title[p_dvd->i_program_chain-1].title.i_cell_nb;
-        DVDFindCell( p_dvd );
+
+        if( DVDFindCell( p_dvd ) < 0 )
+        {
+            intf_ErrMsg( "dvd error: can't find title end" );
+            p_input->b_error = 1;
+            return -1;
+        }
 
         /* temporary hack to fix size in some dvds */
         if( p_dvd->i_cell >= vts.cell_inf.i_cell_nb )
         {
             p_dvd->i_cell = vts.cell_inf.i_cell_nb - 1;
         }
-
+	
+			
         p_dvd->i_sector = 0;
         p_dvd->i_size = DVD_LB_SIZE *
           (off_t)( vts.cell_inf.p_cell_map[p_dvd->i_cell].i_end_sector );
-
-        DVDChapterSelect( p_dvd, 1 );
+        intf_WarnMsg( 2, "dvd info: stream size 1: %lld @ %d", p_dvd->i_size,vts.cell_inf.p_cell_map[p_dvd->i_cell].i_end_sector );
+	
+        if( DVDChapterSelect( p_dvd, 1 ) < 0 )
+        {
+            intf_ErrMsg( "dvd error: can't find first chapter" );
+            p_input->b_error = 1;
+            return -1;
+        }
 
         p_dvd->i_size -= (off_t)( p_dvd->i_sector + 1 ) *DVD_LB_SIZE;
 
@@ -575,8 +631,11 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
         p_es->i_stream_id = 0xe0;
         p_es->i_type = MPEG2_VIDEO_ES;
         p_es->i_cat = VIDEO_ES;
-        input_SelectES( p_input, p_es );
-        intf_WarnMsg( 1, "dvd info: video MPEG2 stream" );
+        intf_WarnMsg( 1, "dvd info: video mpeg2 stream" );
+        if( p_main->b_video )
+        {
+            input_SelectES( p_input, p_es );
+        }
 
         /* Audio ES, in the order they appear in .ifo */
             
@@ -679,29 +738,35 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
             }
         }
 
-        /* For audio: first one if none or a not existing one specified */
-        i_audio = main_GetIntVariable( INPUT_CHANNEL_VAR, 1 );
-        if( i_audio < 0 || i_audio > vts.manager_inf.i_audio_nb )
+        if( p_main->b_audio )
         {
-            main_PutIntVariable( INPUT_CHANNEL_VAR, 1 );
-            i_audio = 1;
+            /* For audio: first one if none or a not existing one specified */
+            i_audio = main_GetIntVariable( INPUT_CHANNEL_VAR, 1 );
+            if( i_audio < 0 || i_audio > vts.manager_inf.i_audio_nb )
+            {
+                main_PutIntVariable( INPUT_CHANNEL_VAR, 1 );
+                i_audio = 1;
+            }
+            if( i_audio > 0 && vts.manager_inf.i_audio_nb > 0 )
+            {
+                input_SelectES( p_input, p_input->stream.pp_es[i_audio] );
+            }
         }
-        if( i_audio > 0 && vts.manager_inf.i_audio_nb > 0 )
+
+        if( p_main->b_video )
         {
-            input_SelectES( p_input, p_input->stream.pp_es[i_audio] );
-        }
-    
-        /* for spu, default is none */
-        i_spu = main_GetIntVariable( INPUT_SUBTITLE_VAR, 0 );
-        if( i_spu < 0 || i_spu > vts.manager_inf.i_spu_nb )
-        {
-            main_PutIntVariable( INPUT_CHANNEL_VAR, 1 );
-            i_spu = 0;
-        }
-        if( i_spu > 0 && vts.manager_inf.i_spu_nb > 0 )
-        {
-            i_spu += vts.manager_inf.i_audio_nb;
-            input_SelectES( p_input, p_input->stream.pp_es[i_spu] );
+            /* for spu, default is none */
+            i_spu = main_GetIntVariable( INPUT_SUBTITLE_VAR, 0 );
+            if( i_spu < 0 || i_spu > vts.manager_inf.i_spu_nb )
+            {
+                main_PutIntVariable( INPUT_SUBTITLE_VAR, 0 );
+                i_spu = 0;
+            }
+            if( i_spu > 0 && vts.manager_inf.i_spu_nb > 0 )
+            {
+                i_spu += vts.manager_inf.i_audio_nb;
+                input_SelectES( p_input, p_input->stream.pp_es[i_spu] );
+            }
         }
     } /* i_title >= 0 */
     else
@@ -716,7 +781,12 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
     if( ( p_area->i_part > 0 ) &&
         ( p_area->i_part <= p_area->i_part_nb ) )
     {
-        DVDChapterSelect( p_dvd, p_area->i_part );
+        if( DVDChapterSelect( p_dvd, p_area->i_part ) < 0 )
+        {
+            intf_ErrMsg( "dvd error: can't set chapter in area" );
+            p_input->b_error = 1;
+            return -1;
+        }
 
         p_input->stream.p_selected_area->i_tell = p_dvd->i_start -
                                                   p_area->i_start;
@@ -729,6 +799,7 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 #undef vts
 #undef vmg
+
     return 0;
 }
 
@@ -738,6 +809,7 @@ static int DVDSetArea( input_thread_t * p_input, input_area_t * p_area )
 static void DVDInit( input_thread_t * p_input )
 {
     thread_dvd_data_t *  p_dvd;
+    input_area_t *       p_area;
     int                  i_title;
     int                  i_chapter;
     int                  i;
@@ -762,13 +834,12 @@ static void DVDInit( input_thread_t * p_input )
      * when using input_ToggleES
      * who wrote thez damn buggy piece of shit ??? --stef */
     p_dvd->i_block_once = 1;//32;
-    p_input->i_read_once = 4;//128;
+    p_input->i_read_once = 8;//128;
 
     i = CSSTest( p_input->i_handle );
 
     if( i < 0 )
     {
-        intf_ErrMsg( "css error: could not get copyright bit" );
         free( p_dvd );
         p_input->b_error = 1;
         return;
@@ -783,10 +854,17 @@ static void DVDInit( input_thread_t * p_input )
         DVDNetlistInit( 2048, 8192, 2048, DVD_LB_SIZE, p_dvd->i_block_once );
     intf_WarnMsg( 2, "dvd info: netlist initialized" );
 
-    /* Ifo initialisation */
-    if( IfoInit( &p_dvd->p_ifo, p_input->i_handle ) < 0 )
+    /* Ifo allocation & initialisation */
+    if( IfoCreate( p_dvd ) < 0 )
     {
-        intf_ErrMsg( "dvd error: fatal failure in IFO" );
+        intf_ErrMsg( "dvd error: allcation error in ifo" );
+        p_input->b_error = 1;
+        return;
+    }
+
+    if( IfoInit( p_dvd->p_ifo ) < 0 )
+    {
+        intf_ErrMsg( "dvd error: fatal failure in ifo" );
         free( p_dvd );
         p_input->b_error = 1;
         return;
@@ -798,38 +876,41 @@ static void DVDInit( input_thread_t * p_input )
         p_dvd->p_css = malloc( sizeof(css_t) );
         if( p_dvd->p_css == NULL )
         {
-            intf_ErrMsg( "dvd error: couldn't create CSS structure" );
+            intf_ErrMsg( "dvd error: couldn't create css structure" );
             free( p_dvd );
             p_input->b_error = 1;
             return;
         }
 
-        p_dvd->p_css->i_fd = p_input->i_handle;
         p_dvd->p_css->i_agid = 0;
 
-        if( CSSInit( p_dvd->p_css ) )
+        if( CSSInit( p_input->i_handle, p_dvd->p_css ) < 0 )
         {
-            intf_ErrMsg( "dvd error: fatal failure in CSS" );
+            intf_ErrMsg( "dvd error: fatal failure in css" );
             free( p_dvd->p_css );
             free( p_dvd );
             p_input->b_error = 1;
             return;
         }
 
-        intf_WarnMsg( 2, "dvd info: CSS initialized" );
+        intf_WarnMsg( 2, "dvd info: css initialized" );
     }
 
+    /* Set stream and area data */
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+
     /* Initialize ES structures */
     input_InitStream( p_input, sizeof( stream_ps_data_t ) );
 
-    /* Set stream and area data */
-    vlc_mutex_lock( &p_input->stream.stream_lock );
+    /* disc input method */
+    p_input->stream.i_method = INPUT_METHOD_DVD;
 
 #define title_inf p_dvd->p_ifo->vmg.title_inf
     intf_WarnMsg( 2, "dvd info: number of titles: %d", title_inf.i_title_nb );
 
 #define area p_input->stream.pp_areas
-    /* We start from 1 here since area 0 is reserved for video_ts.vob */
+    /* We start from 1 here since the default area 0
+     * is reserved for video_ts.vob */
     for( i = 1 ; i <= title_inf.i_title_nb ; i++ )
     {
         input_AddArea( p_input );
@@ -853,8 +934,6 @@ static void DVDInit( input_thread_t * p_input )
     }   
 #undef area
 
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
-
     /* Get requested title - if none try the first title */
     i_title = main_GetIntVariable( INPUT_TITLE_VAR, 1 );
     if( i_title <= 0 || i_title > title_inf.i_title_nb )
@@ -871,8 +950,12 @@ static void DVDInit( input_thread_t * p_input )
 
     p_input->stream.pp_areas[i_title]->i_part = i_chapter;
 
+    p_area = p_input->stream.pp_areas[i_title];
+
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+
     /* set title, chapter, audio and subpic */
-    DVDSetArea( p_input, p_input->stream.pp_areas[i_title] );
+    DVDSetArea( p_input, p_area );
 
     return;
 }
@@ -893,7 +976,7 @@ static void DVDEnd( input_thread_t * p_input )
         free( p_dvd->p_css );
     }
 
-    IfoEnd( p_dvd->p_ifo );
+    IfoDestroy( p_dvd->p_ifo );
     free( p_dvd );
     DVDNetlistEnd( p_netlist );
 }
@@ -930,7 +1013,7 @@ static int DVDRead( input_thread_t * p_input,
     /* Get an iovec pointer */
     if( ( p_vec = DVDGetiovec( p_netlist ) ) == NULL )
     {
-        intf_ErrMsg( "DVD: read error" );
+        intf_ErrMsg( "dvd error: can't get iovec" );
         return -1;
     }
 
@@ -1069,7 +1152,7 @@ static int DVDRead( input_thread_t * p_input,
     vlc_mutex_lock( &p_input->stream.stream_lock );
 
     p_input->stream.p_selected_area->i_tell += i_read_bytes;
-    b_eof = p_input->stream.p_selected_area->i_tell < p_dvd->i_size ? 0 : 1;
+    b_eof = !( p_input->stream.p_selected_area->i_tell < p_dvd->i_size );
 
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
@@ -1132,9 +1215,16 @@ static void DVDSeek( input_thread_t * p_input, off_t i_off )
     p_dvd->i_cell = 0;
 
     /* Find first title cell which is inside program cell */
-    DVDFindCell( p_dvd );
+    if( DVDFindCell( p_dvd ) < 0 )
+    {
+	/* no following cell : we're at eof */
+        intf_ErrMsg( "dvd error: cell seeking failed" );
+        p_input->b_error = 1;
+        return;
+    }
 
     i_cell = p_dvd->i_cell;
+
 #define cell p_dvd->p_ifo->vts.cell_inf.p_cell_map[i_cell]
     /* parse cell address map to find title cell containing sector */
     while( cell.i_end_sector < p_dvd->i_sector )
