@@ -3,8 +3,6 @@
 
 #include "kvlc.moc"
 
-//#include <kmsgbox.h>
-
 KVlc::KVlc() : KTMainWindow()
 {
     btnHello = new QPushButton("Sarass", this);
@@ -18,14 +16,10 @@ KVlc::KVlc() : KTMainWindow()
     connect(btnExit, SIGNAL(clicked()), this, SLOT(slotExit()));
 
     file = new QPopupMenu();
-    file->insertItem("&Wazaaa", this, SLOT(slotHello()));
     file->insertItem("&Exit", this, SLOT(slotExit()));
 
- //   help = kapp->getHelpMenu( TRUE, VOUT_TITLE " (Help Menu)" );
-
     menu = new KMenuBar(this);
     menu->insertItem("&File", file);
- //   menu->insertItem("&Help", help);
 
 }
 
@@ -34,12 +28,8 @@ void KVlc::closeEvent(QCloseEvent *)
     kapp->quit();
 }
 
-void KVlc::slotHello()
-{
-   // KMsgBox::message(0,"Important","Hello World!");
-}
-
 void KVlc::slotExit()
 {
     close();
 }
+
