@@ -4,8 +4,9 @@
  * interface, such as command line.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
+ * $Id: interface.c,v 1.72 2001/04/11 02:01:24 henri Exp $
  *
- * Authors:
+ * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +47,7 @@
 #include "intf_msg.h"
 #include "interface.h"
 #include "intf_cmd.h"
-#include "intf_plst.h"
+#include "intf_playlist.h"
 #include "intf_channels.h"
 #include "keystrokes.h"
 
@@ -155,7 +156,7 @@ static void intf_Manage( intf_thread_t *p_intf )
     if( p_intf->p_input == NULL && !p_intf->b_die )
     {
         /* Select the next playlist item */
-        intf_PlstNext( p_main->p_playlist );
+        intf_PlaylistNext( p_main->p_playlist );
 
         if( p_main->p_playlist->i_index == -1 )
         {
@@ -306,6 +307,7 @@ void intf_AssignNormalKeys( intf_thread_t *p_intf)
     intf_AssignKey( p_intf , ' ', INTF_KEY_TOGGLE_INTERFACE, 0);
     intf_AssignKey( p_intf , 'i', INTF_KEY_TOGGLE_INFO, 0);
     intf_AssignKey( p_intf , 's', INTF_KEY_TOGGLE_SCALING, 0);
+    intf_AssignKey( p_intf , 'd', INTF_KEY_DUMP_STREAM, 0);
 }   
 
 /*****************************************************************************
@@ -320,16 +322,18 @@ int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
     keyparm k_reply;
     
     k_reply = intf_GetKey( p_intf, g_key); 
-    
     switch( k_reply.key )
     {
-    case INTF_KEY_QUIT:                                                  /* quit order */
+    case INTF_KEY_QUIT:                                        /* quit order */
         p_intf->b_die = 1;
         break;
     case INTF_KEY_SET_CHANNEL:
         /* Change channel - return code is ignored since SelectChannel displays
          * its own error messages */
-        intf_SelectChannel( p_intf, k_reply.param );
+/*        intf_SelectChannel( p_intf, k_reply.param ); */
+/*        network_ChannelJoin() */
+/* FIXME : keyboard event is for the time being half handled by the interface
+ * half handled directly by the plugins. We should decide what to do. */        
         break;
     case INTF_KEY_INC_VOLUME:                                    /* volume + */
         if( (p_main->p_aout != NULL) && (p_main->p_aout->vol < VOLUME_MAX) )
@@ -364,10 +368,17 @@ int intf_ProcessKey( intf_thread_t *p_intf, int g_key )
             p_main->p_vout->i_changes |= VOUT_GAMMA_CHANGE;
         }
         break;
+   case INTF_KEY_DUMP_STREAM:
+        if( p_intf->p_input != NULL )
+        {
+            vlc_mutex_lock( &p_intf->p_input->stream.stream_lock );
+            input_DumpStream( p_intf->p_input );
+            vlc_mutex_unlock( &p_intf->p_input->stream.stream_lock );
+        }
+        break;
    default:                                                   /* unknown key */
         return( 1 );
     }
 
     return( 0 );
 }
-
